# Un beau jeu d'Othello

## Démarrage du programme

Le programme dépend du package pygame. Dans le répertoire contenant le fichier `__main__.py`, faire la commande `python .`.

## Auteurs

- Petros Fytilis
- Charlotte Foucard

## Fonctionnalités

- Permet de jouer sur une planche 6x6, 8x8 ou 10x10.

- Permet de jouer humain contre humain, humain contre ordinateur et ordinateur contre ordinateur. Nous vous suggérons d'essayer une partie ordinateur contre ordinateur, c'est très satisfaisant.

- Implémentation d'un joueur ordinateur à difficulté variable :
	- Facile : joue aléatoirement ;
	- Moyen⁴ : emploie un algorithme minimax¹² (profondeur 2) avec l'élagage alpha-bêta³ et évaluation par tableau⁴ ;
	- Difficile : emploie un algorithme minimax (profondeur 4) avec l'élagage alpha-bêta et évaluation par tableau.

		¹ L'algorithme minimax est « un algorithme qui s'applique à la théorie des jeux pour les jeux à deux joueurs à somme nulle (et à information complète) consistant à minimiser la perte maximum ». Nous construisons un arbre représentant toutes les possibilités de jeu pour n itérations (soit la profondeur). À chaque itération nous alternons entre le joueur courant (qui essaye de maximiser son score) et son adversaire (qui essaye de minimiser le score au joueur courant), puis nous évaluons l'état de la planche lorsqu'un nœud feuille est atteint.

		² L’algorithme minimax est amélioré avec l'élagage alpha-bêta qui nous permet de réduire les branches de l’arbre qui doivent être parcourues. Ceci réduit le temps d’exécution de l’ordinateur approximativement de 80%. Il utilise deux variables, soit alpha (qui représente la valeur maximale connue jusqu’à présent, initialisée à -∞) et bêta (qui représente la valeur minimale connue jusqu’à présent, initialisée à ∞). Lorsqu’alpha surpasse bêta, l’algorithme peut quitter la branche courante pour réduire le temps d’exécution sans pourtant affecter les résultats.

		³ Nous construisons une table de pointage selon la dimension de la planche. Chaque case est attribuée à un pointage selon l'importance de la case. Par exemple les coins sont de haute importance, tandis que les cases adjacentes aux coins sont de faible importance. L'évaluation est effectuée en comparant l'état de la planche avec le tableau à évaluation.

		EXAMPLE DE TABLE DE POINTAGE POUR dimension_planche == 8:
		```
		[[100,  -1,  5,  2,  2,  5,  -1, 100],
		[  -1, -20,  1,  1,  1,  1, -20,  -1],
		[   5,   1,  1,  1,  1,  1,   1,   5],
		[   2,   1,  1,  0,  0,  1,   1,   2],
		[   2,   1,  1,  0,  0,  1,   1,   2],
		[   5,   1,  1,  1,  1,  1,   1,   5],
		[  -1, -20,  1,  1,  1,  1, -20,  -1],
		[ 100,  -1,  5,  2,  2,  5,  -1, 100]]
		```

		⁴ Nous avions essayez un algorithme qui sélectionne le coup avec le plus de pièces mangées, cependant cette stratégie n'était pas capable de vaincre l'ordinateur facile avec consistance.

- Multithreading (1 thread pour l’interface graphique, 1 thread pour l'animation et 1 thread pour chaque joueur ordinateur). Nous utilisons le multithreading pour que l’interface graphique puisse continuer de fonctionner en parallèle lors du processus de sélection de coup des joueurs ordinateurs. Nous utilisons un "lock" pour empêcher plus d'un joueur d’interagir avec la partie à la fois.

- Implémentation d’une option astuce : lorsque le joueur humain a besoin d’un coup de main, il peut sélectionner l’option astuce qui lui suggère un coup jugé comme étant optimal selon notre algorithme minimax à profondeur 4.

- Implémentation d’une option « retour en arrière » : lorsque le joueur humain souhaite revenir sur ses décisions il peut sélectionner l’option retour pour revenir en arrière jusqu’à son dernier coup joué. Pour ce faire, nous avons implémenté une pile qui store tous les coups joués et les pièces mangées à chaque tour. Pour retourner en arrière nous dépilons le dernier élément de la pile puis renversons le coup joué.

## Esthétique

- Aspect sonore (musique de fond et ajout de pièce) grâce au module pygame. Pygame est compatible sur Windows et Linux et permet de superposer diverses trames sonores.

- Permet à l'utilisateur de chosir entre 5 arrière-plans et de faire une rotation en pleine partie. Choisi un arrière-plan aléatoire à l'ouverture de la fenêtre.

- Permet au joueur de sélectionner un de trois jetons personnalisés par couleur.

- Affichage d'un compteur dynamique des pièces noires et des pièces blanches.

- Affichage des coups possibles combiné avec le nombre de pièces pouvant être mangées à chaque case.

- Affichage clair de la pièce dernièrement jouée.

- Nous utilisons la fonction wait() du module time lors des coups de joueur ordinateur pour que l'utilisateur aille le temps de traiter le déroulement du jeu. Sinon tout ce passe beaucoup trop vite pour l'oeil humain.

- Options pour afficher les règles du jeu et les crédits.

- Compatible et testé sur Windows 8, Windows 10 et Ubuntu.

- Sélectionne la police de texte dépendamment du système d'opération de l'utilisateur.

- Image d'avatar animée. Nous utilisons un thread unique pour gérer l’animation de l’avatar. Ce thread alterne entre deux objets PhotoImage pour afficher une animation visuelle. Il s’exécute dans une boucle infinie.
