class Piece:
    """
    Classe modélisant une pièce d'un jeu d'Othello.
    """
    def __init__(self, couleur, rangee, colonne):
        """
        Constructeur d'une nouvelle pièce.

        Args:
            couleur: couleur de la pièce, soit "blanc" ou "noir".
            rangee: rangée de la pièce, de 0 à la dimension de la planche.
            colonne: colonne de la pièce, de 0 à la dimension de la planche.
        """
        assert couleur in ["blanc", "noir"], "Piece: couleur invalide."
        self.couleur = couleur
        self.rangee = rangee
        self.colonne = colonne

    def est_blanc(self):
        """
        Retourne si la pièce est de couleur blanche.

        Returns:
            True si la pièce est de couleur blanche, False autrement.
        """
        return self.couleur == "blanc"

    def est_noir(self):
        """
        Retourne si la pièce est de couleur noire.

        Returns:
            True si la pièce est de couleur noire, False autrement.
        """
        return self.couleur == "noir"

    def echange_couleur(self):
        """
        Fait le changement de couleur de la pièce lorsqu'elle est mangée.
        """
        self.couleur = "noir" if self.est_blanc() else "blanc"

    def __repr__(self):
        """
        Faire un print(une_piece) affichera un caractère unicode représentant le dessin d'une pièce.
        """
        if self.est_noir():
            return "\u265e"
        else:
            return "\u2730"
